
/*
*  keyfile_io - an easy way to read and write keyfiles
*  Copyright (c) 2004-2006 by Mattias Hultgren <mattias_hultgren@tele2.se>
*
*  See vartypes.h
*/

#include <stdio.h>
#include <fstream>
#include <string.h>
#include "vartypes.h"

keyfile_io::keysDB::keysDB(keysDB &src)
{
	*this = src;
}
void keyfile_io::keysDB::operator=(keysDB &src)
{
	name = src.name;
	value = src.value;
}


keyfile_io::keyfile_io()
{
	keys = 0;
	nrofkeys = 0;
	replace_escape_sequences = false;
}
keyfile_io::keyfile_io(const char *filename) throw(error_obj)
{
	keys = 0;
	nrofkeys = 0;
	replace_escape_sequences = false;

	read( filename );
}
keyfile_io::keyfile_io(const keyfile_io &src) throw(error_obj)
{
	*this = keyfile_io();
	*this = src;
}

keyfile_io::~keyfile_io()
{
	reset();
}

void keyfile_io::operator=(const keyfile_io &src)
{
	printf("\n\nkeyfile_io::operator() not implemented\n\n");
	exit(-1);
}

void keyfile_io::reset(void)
{
	delete [] keys;
	keys = 0;
	nrofkeys = 0;
}

void keyfile_io::read_intern(const char *intern_file) throw(error_obj)
{
	FILE *fp;

	fp = tmpfile();
	if(fp == 0)
		Throw_Couldnt_open_write_to_file( "<tmp_file>" );

	if(fwrite( intern_file, 1, strlen(intern_file), fp ) != strlen(intern_file))
	{
		fclose(fp);
		Throw_Couldnt_open_write_to_file( "<tmp_file>" );
	}

	rewind( fp );

	try
	{
		read( fp, "<tmp_file>" );
	}
	catch(...)
	{
		fclose( fp );
		throw;
	}
	if(fclose(fp) != 0)
		Throw_Couldnt_close_file( "<tmp_file>" );
}

void keyfile_io::read(const char *filename) throw(error_obj)
{
	FILE *fp;

	fp = fopen(filename,"r");
	if(fp == 0)
		Throw_Couldnt_open_read_from_file( filename );

	try
	{
		read( fp, filename );
	}
	catch(...)
	{
		fclose( fp );
		throw;
	}
	if(fclose(fp) != 0)
		Throw_Couldnt_close_file( filename );
}

void keyfile_io::read(FILE *open_file, const char *filename) throw(error_obj)
{
	char str[5010];
	int i,keyname,value,equal,i2;

	for(;;)
	{
		if(fgets(str,5005,open_file) == 0)
		{
			if(feof(open_file)) // this is not an error, the hole file is read
				break;

			Throw_Couldnt_open_read_from_file( filename );
		}
		str[5007] = ' ';
		str[5008] = '\"';
		str[5009] = '\0'; // this is to prevent that the loops overruns the buffer limit

		for(i=0; str[i] == ' ' || str[i] == '\t' ;i++); // skipping all spaces and tabs at the beginning of the row

		if(str[i] == '#') // this line is a comment
			continue;

		if( str[i] == '\n'  ||  str[i] == '\r'  ||  str[i] == '\0' ) // empty line
			continue;

		keyname = i;

		if( str[i] == '\"' )
		{
			keyname++;
			for(i++;;i++)
			{
				if( str[i] == '\"' )
				{
					// let's check if this " is escaped out
					for( i2=0; str[i-i2-1] == '\\'; i2++ ) ;
					
					if( i2 > 0  &&  (i2%2) == 1 ) // check if the \ is escaped out
						continue;

					str[i] = 0;
					break;
				}
			}
		}
		else
		{
			for(;;i++)
			{
				if( !( ((str[i] >= 'a') & (str[i] <= 'z')) | ((str[i] >= 'A') & (str[i] <= 'Z')) | (str[i] == '_') |
				       ((str[i] >= '0') & (str[i] <= '9')) )  )
				{
					if(str[i] != ' ' || str[i] == '\t')
						Throw_Invalid_format_in_file( filename );
					str[i] = 0;
					break;
				}
			}
		}

		for(equal=0,i++;;i++) // getting start of value
		{
			if(str[i] == ' ' || str[i] == '\t')
				{  }
			else if(str[i] == '=')
			{
				if(equal == 1) // more than one '=' sign
					Throw_Invalid_format_in_file( filename );
				equal = 1;
			}
			else if(str[i] == '\"')
			{
				if(equal == 0)
					Throw_Invalid_format_in_file( filename );
				i++;
				break;
			}
			else
				Throw_Invalid_format_in_file( filename );
		}
		value = i;

		for(;;i++) // getting the end of value
		{
			if(str[i] == '\"')
			{
				// let's check if this " is escaped out
				for( i2=0; str[i-i2-1] == '\\'; i2++ ) ;
				
				if( i2 > 0  &&  (i2%2) == 1 ) // check if the \ is escaped out
					continue;

				break;
			}
			else if(str[i] == 0)
				Throw_Invalid_format_in_file( filename );
		}
		str[i] = 0;

		set_key( &str[keyname], &str[value] );
	}
}

void keyfile_io_replace_escape_sequences(std::string &str) throw(error_obj)
{
	for(unsigned int ind=0;ind < str.size();ind++)
	{
		if(str[ind] == '\\')
		{
			switch(str[ind+1])
			{
			case '\\':
				str.replace( ind, 2, "\\" );
				break;
			case 'n':
				str.replace( ind, 2, "\n" );
				break;
			case 'r':
				str.replace( ind, 2, "\r" );
				break;
			case 't':
				str.replace( ind, 2, "\t" );
				break;
			case '\"':
				str.replace( ind, 2, "\"" );
				break;

			default:
				THROW_MACRO_FIX( 22, "Unknown escape sequence" );
			}
		}
	}
}

void keyfile_io::set_key(const std::string &keyname,const std::string &value) throw(error_obj)
{
	std::string tmp_keyname( keyname );
	keysDB *newkeys = 0;
	
	if(replace_escape_sequences)
		keyfile_io_replace_escape_sequences( tmp_keyname );
	

	for(uint32 i=0;i<nrofkeys;i++)
	{
		if(keys[i].name == tmp_keyname) // key already exists, replace it
		{
			try
			{
				keys[i].value = value;

				if(replace_escape_sequences)
					keyfile_io_replace_escape_sequences( keys[i].value );

				return;
			}
			catch(error_obj) { throw; }
			catch(...) { Throw_Couldnt_get_memory(); }
		}
	}

	try
	{
		newkeys = new keysDB [ nrofkeys + 1 ];

		newkeys[nrofkeys].name = tmp_keyname;
		newkeys[nrofkeys].value = value;
	}
	catch(...)
	{
		if(newkeys != 0)
			delete [] newkeys;

		Throw_Couldnt_get_memory();
	}

	if(replace_escape_sequences)
		keyfile_io_replace_escape_sequences( newkeys[nrofkeys].value );

	for(uint32 i=0;i<nrofkeys;i++)
		newkeys[i] = keys[i];

	delete [] keys;
	keys = newkeys;
	nrofkeys++;
}

void keyfile_io::remove_key(const std::string &keyname) throw(error_obj)
{
	keysDB *newkeys = 0;

	for(uint32 i=0;i<nrofkeys;i++)
	{
		if(keys[i].name == keyname) // key found
		{
			if(nrofkeys == 1)
			{
				reset();
				return;
			}
			else
			{
				try
				{
					newkeys = new keysDB [ nrofkeys - 1 ];
				}
				catch(error_obj) { throw; }
				catch(...) { Throw_Couldnt_get_memory(); }

				uint32 step = 0;
				for(uint32 o=0;o<nrofkeys;o++)
				{
					if(o == i)
						step = 1;
					else
						newkeys[o - step] = keys[o];
				}
				delete [] keys;
				keys = newkeys;
				nrofkeys--;
			}
		}
	}
}

const std::string * keyfile_io::get_key(const std::string &keyname, const std::string *fail_pointer)
{
	for(uint32 i=0;i<nrofkeys;i++)
	{
		if(keys[i].name == keyname)
			return &keys[i].value;
	}
	return fail_pointer;
}
const char * keyfile_io::get_key_c_str(const std::string &keyname, const char *fail_pointer)
{
	for(uint32 i=0;i<nrofkeys;i++)
	{
		if(keys[i].name == keyname)
			return keys[i].value.c_str();
	}
	return fail_pointer;
}

const char* keyfile_io::get_text( const char *keyname )
{
	for(uint32 i=0;i<nrofkeys;i++)
	{
		if(keys[i].name == keyname)
			return keys[i].value.c_str();
	}
	return keyname;
}

void keyfile_io_expand_to_escape_sequences( const std::string &src, std::string &dest )
       throw(error_obj)
{
	dest = src;
	
	try
	{
		for( unsigned int i=0;i < dest.size(); i++ )
		{
			switch( dest[i] )
			{
			case '\\':
				dest.replace( i, 1, "\\\\" );
				i++;
				break;
			case '\n':
				dest.replace( i, 1, "\\n" );
				i++;
				break;
			case '\r':
				dest.replace( i, 1, "\\r" );
				i++;
				break;
			case '\t':
				dest.replace( i, 1, "\\t" );
				i++;
				break;
			case '\"':
				dest.replace( i, 1, "\\\"" );
				i++;
				break;

			default:
				break;
			}
		}
	}
	catch(...) {  Throw_Couldnt_get_memory();  }
}

void keyfile_io::write(const char *filename) throw(error_obj)
{
	std::ofstream file;

	file.open( filename );

	if( !file.is_open() )
		Throw_Couldnt_open_write_to_file( filename );

	if( replace_escape_sequences )
	{
		std::string name, value;
		
		for( uint32 i=0; i<nrofkeys ;i++ )
		{
			keyfile_io_expand_to_escape_sequences( keys[i].name, name );
			keyfile_io_expand_to_escape_sequences( keys[i].value, value );
			file << name << " = \"" << value << "\"\n";
		}
	}
	else
	{
		for( uint32 i=0; i<nrofkeys ;i++ )
			file << keys[i].name << " = \"" << keys[i].value << "\"\n";
	}

	file.close();
	if( file.is_open() )
		Throw_Couldnt_close_file( filename );
}
