

/*
 *  Math2
 *  Copyright (c) 2003-2006 by Mattias Hultgren <mattias_hultgren@tele2.se>
 *
 *  See math2.h
 */


#include "math2.h"
#include "math2.intern.h"

namespace math
{

class CodeBlockFunction
{
public:
	std::string name;
	CodeBlock *function;
	Vector<ArgumentType> arguments;

	CodeBlockFunction() { name = ""; function = 0; }
	CodeBlockFunction(const CodeBlockFunction &src) throw(error_obj) { name = ""; function = 0; *this = src; }
	~CodeBlockFunction() { delete function; }

	void operator=(const CodeBlockFunction &src) throw(error_obj);
};

void CodeBlockFunction::operator=(const CodeBlockFunction &src) throw(error_obj)
{
	printf("\nCodeBlockFunction::operator=() not implemented yet\n");
	exit(-1);
}

Vector<CodeBlockFunction> code_block_functions;


FunctionList function_list;


void wrapper_check_argument( int i, int i2, const Variable *right ) throw(error_obj)
{
	Variable *var;
	const Variable *tmp;
	int max = code_block_functions[i]->arguments.get_size();
	var = code_block_functions[i]->function->private_varlist.get_pointer_rw( 
	        code_block_functions[i]->function->private_varlist.get_id( 
	        code_block_functions[i]->arguments[i2]->variable_name.c_str()) );

	switch( code_block_functions[i]->arguments[i2]->type )
	{
	case reserved_variable:
		if( max == 1 )
			*var = *right;
		else
			*var = *( right->get_array()->get_variable(i2) );
		break;
		
	case reserved_integer:
		tmp = (max == 1) ? right : right->get_array()->get_variable(i2);
		if( tmp->get_type() == VariableType_Complex )
		{
			if( tmp->get_complex()->imaginary == Real(0.0) )
			{
				if(  tmp->get_complex()->real.isInteger() )
					*var = *tmp;
				else
					Throw_Data_type_is_unsupported_by( code_block_functions[i]->name.c_str() );
			}
			else
				Throw_Data_type_is_unsupported_by( code_block_functions[i]->name.c_str() );
		}
		else
			Throw_Data_type_is_unsupported_by( code_block_functions[i]->name.c_str() );
		break;

	case reserved_real:
		tmp = (max == 1) ? right : right->get_array()->get_variable(i2);
		if( tmp->get_type() == VariableType_Complex )
		{
			if( tmp->get_complex()->imaginary == Real(0.0) )
				*var = *tmp;
			else
				Throw_Data_type_is_unsupported_by( code_block_functions[i]->name.c_str() );
		}
		else
			Throw_Data_type_is_unsupported_by( code_block_functions[i]->name.c_str() );
		break;

	case reserved_complex:
		tmp = (max == 1) ? right : right->get_array()->get_variable(i2);
		if( tmp->get_type() == VariableType_Complex )
			*var = *tmp;
		else
			Throw_Data_type_is_unsupported_by( code_block_functions[i]->name.c_str() );
		break;

	case reserved_string:
		tmp = (max == 1) ? right : right->get_array()->get_variable(i2);
		if( tmp->get_type() == VariableType_String )
			*var = *tmp;
		else
			Throw_Data_type_is_unsupported_by( code_block_functions[i]->name.c_str() );
		break;

	case reserved_matrix:
		tmp = (max == 1) ? right : right->get_array()->get_variable(i2);
		if( tmp->get_type() == VariableType_Matrix )
			*var = *tmp;
		else
			Throw_Data_type_is_unsupported_by( code_block_functions[i]->name.c_str() );
		break;

	case reserved_picture:
		tmp = (max == 1) ? right : right->get_array()->get_variable(i2);
		if( tmp->get_type() == VariableType_Picture )
			*var = *tmp;
		else
			Throw_Data_type_is_unsupported_by( code_block_functions[i]->name.c_str() );
		break;

	case reserved_array:
		tmp = (max == 1) ? right : right->get_array()->get_variable(i2);
		if( tmp->get_type() == VariableType_Array )
			*var = *tmp;
		else
			Throw_Data_type_is_unsupported_by( code_block_functions[i]->name.c_str() );
		break;

	case reserved_boolean:
		tmp = (max == 1) ? right : right->get_array()->get_variable(i2);
		if( tmp->get_type() == VariableType_Boolean )
			*var = *tmp;
		else
			Throw_Data_type_is_unsupported_by( code_block_functions[i]->name.c_str() );
		break;

	default:
		Throw_Data_type_is_unsupported_by( "wrapper_code_block_function, internal error, please report it" );
	}
}

void wrapper_code_block_functions( Variable *res, Variable *left,const Variable *right,
                                         const std::string &name ) throw(error_obj)
{
	int i;

	for( i=0; i<code_block_functions.get_size(); i++ )
	{
		if( code_block_functions[i]->name == name )
			break;
	}

	if( i == code_block_functions.get_size() )
		THROW_MACRO_FIX( 22, "Unknown function" ); // name wasn't found

	if( code_block_functions[i]->arguments.get_size() == 0 )
	{
		if( left != 0  ||  right != 0 )
			Throw_Syntax_error();
	}
	else
	{
		if( left != 0  ||  right == 0 )
			Throw_Syntax_error();

		if( code_block_functions[i]->arguments.get_size() == 1 )
			wrapper_check_argument( i, 0, right );
		else
		{
			int i2;

			if( right->get_type() != VariableType_Array )
				Throw_Syntax_error( "too few arguments" );

			if( right->get_array()->get_size() != code_block_functions[i]->arguments.get_size() )
				Throw_Syntax_error( "wrong number of arguments" );

			for( i2=0; i2<code_block_functions[i]->arguments.get_size(); i2++ )
				wrapper_check_argument( i, i2, right );
		}
	}

	code_block_functions[i]->function->calc( res );
}


FunctionList::FunctionList(const FunctionList &src)
{
	list = src.list;
}


void FunctionList::add_function( const std::string &new_name, BuiltinFunctionPtr new_func, int new_priority,
                                 bool need_left, bool need_right, bool new_isdeletable, bool hard_linking ) throw(error_obj)
{
	FunctionListElement new_element;

	check_name_against_reserved(new_name);

	{
		bool found_var = false;
		try
		{
			global_varlist.get_id(new_name.c_str());
			found_var = true;
		}
		catch(...)
		  {  }
		if(found_var)
			Throw_Name_is_not_unique( new_name.c_str() );
	}

	if( check_for( new_name ) )
		Throw_Name_is_not_unique( new_name.c_str() );

	if( new_isdeletable  &&  hard_linking )
		THROW_MACRO_FIX( 0, "Deletable function tries to be added with hard linking." );

	try{  new_element.name = new_name;  }
	catch(...) { Throw_Couldnt_get_memory(); }

	new_element.func = new_func;
	new_element.priority = new_priority;
	new_element.need_left = need_left;
	new_element.need_right = need_right;
	new_element.hard_linking = hard_linking;
	new_element.isdeletable = new_isdeletable;
	new_element.is_code_block_function = false;

	try
	{
		list.append( new_element );
	}
	catch(...) {  Throw_Couldnt_get_memory();  }  
}

void FunctionList::add_this_code_block_function( const std::string &new_name,
                                                 CodeBlock *function, 
                                                 const Vector<ArgumentType> &arguments )
                                                 throw(error_obj)
{
	CodeBlockFunction *tmp = 0;

	add_function( new_name, BuiltinFunctionPtr(wrapper_code_block_functions), 14, false, (arguments.get_size()!=0), true, false );
	set_as_code_block_function( new_name );
	try
	{
		tmp = new CodeBlockFunction;

		tmp->name = new_name;
		tmp->function = function;
		tmp->arguments = arguments;

		code_block_functions.append_this( tmp );
	}
	catch(...)
	{
		delete tmp;
		delete_function( new_name );
		Throw_Couldnt_get_memory();
	}
}


// returns 1 if the function was found and deleted
// return -1 if the function was found but was undeletable
// return 0 if the function wasn't found
int FunctionList::delete_function(const std::string &name)
{
	int i;

	for( i=0; i<list.get_size(); i++ )
	{
		if( list[i]->name == name )
			break;
	}
	if( i == list.get_size() )
		return 0; // name wasn't found

	if(!list[i]->isdeletable)
		return -1; // found but was undeletable

	if( list[i]->is_code_block_function )
	{
		int i2,tmp_size;

		tmp_size = code_block_functions.get_size();
		for( i2=0; i2<tmp_size; i2++ )
		{
			if( code_block_functions[i2]->name == name )
			{
				if( code_block_functions[i2]->function != 0 )
					if( code_block_functions[i2]->function->get_inuse() )
						return -1;

				code_block_functions.remove( i2 );
				break;
			}
		}
		if( i2 == tmp_size )
			THROW_MACRO_FIX( 22, "Unknown function" ); // name wasn't found
	}

	list.remove( i );

	return 1; // was found and deleted
}

// returns true if the function was found and isdeletable-flag is cleared
// returns false if the function wasn't found
bool FunctionList::set_undeletable(const std::string &name)
{
	int i;

	for( i=0; i<list.get_size(); i++ )
	{
		if( list[i]->name == name )
			break;
	}
	if( i == list.get_size() )
		return false; // name wasn't found

	list[i]->isdeletable = false;

	return true;
}

void FunctionList::set_as_code_block_function( const std::string &name )
{
	int i;

	for( i=0; i<list.get_size(); i++ )
	{
		if( list[i]->name == name )
			break;
	}
	if( i == list.get_size() )
		return; // name wasn't found

	list[i]->is_code_block_function = true;
}

// returns true if there is a function with this name, else false is returned
bool FunctionList::check_for(const std::string &name) const
{
	int i;

	for( i=0; i<list.get_size(); i++ )
	{
		if( list[i]->name == name )
			break;
	}
	return ( i != list.get_size() );
}

BuiltinFunctionPtr FunctionList::get( const std::string &name, int *priority, bool *need_left, bool *need_right, bool *hard_linking ) const
{
	int i;

	for( i=0; i<list.get_size(); i++ )
	{
		if( list[i]->name == name )
			break;
	}
	if( i == list.get_size() )
		return 0; // name wasn't found

	*priority = list[i]->priority;
	*need_left = list[i]->need_left;
	*need_right = list[i]->need_right;
	*hard_linking = list[i]->hard_linking;

	return list[i]->func;
}

} // namespace math
