
/*
 *  variable typedef's
 *  Copyright (c) 2002-2005 by Mattias Hultgren <mattias_hultgren@tele2.se>
 *
 *  See vartypes.h
 */

#include "vartypes.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

keyfile_io translation;

#ifdef DONT_HAVE_TRUNC
double trunc(double value)
{
	if( value < 0.0 )
		return -floor( -value );
	return floor( value );
}
#endif
#ifdef DONT_HAVE_LOG2
double log2(double value)
{
	return ( log( value ) / log( 2.0 ) );
}
#endif

#if defined USE_FLOAT  ||  USE_DOUBLE
	extern const floatx PI = M_PI;
	extern const floatx E = M_E;
#endif
#ifdef USE_LONG_DOUBLE
	#ifdef M_PIl
		const floatx PI = M_PIl;
	#else
		const floatx PI = M_PI;
	#endif
	#ifdef M_El
		const floatx E = M_El;
	#else
		const floatx E = M_E;
	#endif
#endif

class vartypes_check
{
public:
	vartypes_check();

}check;

void Throw_Invalid_width_and_or_height(void) throw(error_obj)
{
	error_obj error;
	std::string trans;
	const std::string *tmp;
	const char err[] = "Invalid width and/or height";

	error.errorcode = 1;

	trans = err;
	if( (tmp = translation.get_key("ERROR_INVALID_WIDTH_AND_OR_HEIGHT")) )
		trans = *tmp;
	snprintf(error.msg,ERROR_OBJ_MSG_LEN,"%s",trans.c_str());
	throw error;
}
void Throw_Invalid_matrix_width(void) throw(error_obj)
{
	error_obj error;
	std::string trans;
	const std::string *tmp;
	const char err[] = "Invalid matrix width";

	error.errorcode = 1;

	trans = err;
	if( (tmp = translation.get_key("ERROR_INVALID_MATRIX_WIDTH")) )
		trans = *tmp;
	snprintf(error.msg,ERROR_OBJ_MSG_LEN,"%s",trans.c_str());
	throw error;
}
void Throw_Couldnt_get_memory(void) throw(error_obj)
{
	error_obj error;

	error.errorcode = 2;

	snprintf(error.msg,ERROR_OBJ_MSG_LEN, _("Couldn't get memory") );
	throw error;
}
void Throw_Couldnt_open_read_from_file(const char *filename) throw(error_obj)
{
	error_obj error;

	error.errorcode = 4;

	snprintf( error.msg, ERROR_OBJ_MSG_LEN, _("Couldn't open/read from file (%s)"), filename );
	throw error;
}
void Throw_File_not_a_valid(const char *type, const char *filename) throw(error_obj)
{
	error_obj error;
	std::string trans;
	const std::string *tmp;
	const char err[] = "File_not_a_valid";

	error.errorcode = 5;

	trans = err;
	if( (tmp = translation.get_key("ERROR_FILE_NOT_A_VALID")) )
		trans = *tmp;
	snprintf(error.msg,ERROR_OBJ_MSG_LEN,"%s %s (%s)",trans.c_str(),type,filename);
	throw error;
}
void Throw_Couldnt_open_write_to_file(const char *filename) throw(error_obj)
{
	error_obj error;

	error.errorcode = 7;

	snprintf( error.msg, ERROR_OBJ_MSG_LEN, _("Couldn't open/write to file (%s)"), filename );
	throw error;
}
void Throw_Unexpected(const char *unexpected) throw(error_obj)
{
	error_obj error;
	std::string trans;
	const std::string *tmp;
	const char err[] = "Unexpected";

	error.errorcode = 16;

	trans = err;
	if( (tmp = translation.get_key("ERROR_UNEXPECTED")) )
		trans = *tmp;
	snprintf(error.msg,ERROR_OBJ_MSG_LEN,"%s %s",trans.c_str(),unexpected);
	throw error;
}
void Throw_Expected(const char *expected) throw(error_obj)
{
	error_obj error;
	std::string trans;
	const std::string *tmp;
	const char err[] = "Expected";

	error.errorcode = 17;

	trans = err;
	if( (tmp = translation.get_key("ERROR_EXPECTED")) )
		trans = *tmp;
	snprintf(error.msg,ERROR_OBJ_MSG_LEN,"%s %s",trans.c_str(),expected);
	throw error;
}
void Throw_Divide_by_zero(void) throw(error_obj)
{
	error_obj error;

	error.errorcode = 18;

	snprintf( error.msg, ERROR_OBJ_MSG_LEN, _("Divide by zero") );
	throw error;
}
void Throw_Domain_error(const char *domain_error) throw(error_obj)
{
	error_obj error;
	std::string trans;
	const std::string *tmp;
	const char err[] = "Domain error";

	error.errorcode = 19;

	trans = err;
	if( (tmp = translation.get_key("ERROR_DOMAIN_ERROR")) )
		trans = *tmp;
	snprintf(error.msg,ERROR_OBJ_MSG_LEN,"%s: %s",trans.c_str(),domain_error);
	throw error;
}
void Throw_Illegal_use_of_reserved_key_word(const char *word) throw(error_obj)
{
	error_obj error;
	std::string trans;
	const std::string *tmp;
	const char err[] = "Illegal use of reserved key word";

	error.errorcode = 20;

	trans = err;
	if( (tmp = translation.get_key("ERROR_ILLEGAL_USE_OF_RESERVED_KEY_WORD")) )
		trans = *tmp;
	snprintf(error.msg,ERROR_OBJ_MSG_LEN,"%s (%s)", trans.c_str(), word);
	throw error;
}
void Throw_Syntax_error(const char *syntax_error) throw(error_obj)
{
	error_obj error;
	std::string trans;
	const std::string *tmp;
	const char err[] = "Syntax error";

	error.errorcode = 23;

	trans = err;
	if( (tmp = translation.get_key("ERROR_SYNTAX_ERROR")) )
		trans = *tmp;
	if(syntax_error == 0)
		snprintf(error.msg,ERROR_OBJ_MSG_LEN,"%s",trans.c_str());
	else
		snprintf(error.msg,ERROR_OBJ_MSG_LEN,"%s: %s",trans.c_str(),syntax_error);
	throw error;
}
void Throw_Tried_to_change_a_constant(void) throw(error_obj)
{
	error_obj error;

	error.errorcode = 25;

	snprintf( error.msg, ERROR_OBJ_MSG_LEN, _("Tried to change a constant") );
	throw error;
}
void Throw_Couldnt_close_file(const char *filename) throw(error_obj)
{
	error_obj error;

	error.errorcode = 26;

	snprintf( error.msg, ERROR_OBJ_MSG_LEN, _("Couldn't close file (%s)"), filename );
	throw error;
}
void Throw_Data_type_is_unsupported_by(const char *function) throw(error_obj)
{
	error_obj error;

	error.errorcode = 29;

	snprintf( error.msg, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), function );
	throw error;
}
void Throw_Circle_reference_of_variable(const char *varname) throw(error_obj)
{
	error_obj error;
	std::string trans;
	const std::string *tmp;
	const char err[] = "Cirlce reference of variable";

	error.errorcode = 31;

	trans = err;
	if( (tmp = translation.get_key("ERROR_CIRLCE_REFERENCE_OF_VARIABLE")) )
		trans = *tmp;
	snprintf(error.msg,ERROR_OBJ_MSG_LEN,"%s '%s'",trans.c_str(),varname);
	throw error;
}
void Throw_List_dont_match_for(const char *function) throw(error_obj)
{
	error_obj error;
	std::string trans;
	const std::string *tmp;
	const char err[] = "List don't match for";

	error.errorcode = 32;

	trans = err;
	if( (tmp = translation.get_key("ERROR_LIST_DONT_MATCH_FOR")) )
		trans = *tmp;
	snprintf(error.msg,ERROR_OBJ_MSG_LEN,"%s '%s'",trans.c_str(),function);
	throw error;
}
void Throw_Matrix_size_dont_match(void) throw(error_obj)
{
	error_obj error;
	std::string trans;
	const std::string *tmp;
	const char err[] = "Matrix size don't match";

	error.errorcode = 32;

	trans = err;
	if( (tmp = translation.get_key("ERROR_MATRIX_SIZE_DONT_MATCH")) )
		trans = *tmp;
	snprintf(error.msg,ERROR_OBJ_MSG_LEN,"%s",trans.c_str());
	throw error;
}
void Throw_Overflow(void) throw(error_obj)
{
	error_obj error;

	error.errorcode = 33;

	snprintf( error.msg, ERROR_OBJ_MSG_LEN, _("Overflow") );
	throw error;
}
void Throw_Name_is_not_unique(const char *name) throw(error_obj)
{
	error_obj error;
	std::string trans;
	const std::string *tmp;
	const char err[] = "Name is not unique";

	error.errorcode = 41;

	trans = err;
	if( (tmp = translation.get_key("ERROR_NAME_IS_NOT_UNIQUE")) )
		trans = *tmp;
	snprintf(error.msg,ERROR_OBJ_MSG_LEN,"%s (%s)",trans.c_str(),name);
	throw error;
}
void Throw_No_solution_found(void) throw(error_obj)
{
	error_obj error;
	std::string trans;
	const std::string *tmp;
	const char err[] = "No solution found";

	error.errorcode = 43;

	trans = err;
	if( (tmp = translation.get_key("ERROR_NO_SOLUTION_FOUND")) )
		trans = *tmp;
	snprintf(error.msg,ERROR_OBJ_MSG_LEN,"%s",trans.c_str());
	throw error;
}
void Throw_Invalid_format_in_file(const char *filename) throw(error_obj)
{
	error_obj error;
	std::string trans;
	const std::string *tmp;
	const char err[] = "Invalid format in file";

	error.errorcode = 48;

	trans = err;
	if( (tmp = translation.get_key("ERROR_INVALID_FORMAT_IN_FILE")) )
		trans = *tmp;
	snprintf(error.msg,ERROR_OBJ_MSG_LEN,"%s (%s)",trans.c_str(),filename);
	throw error;
}
void Throw_Old_version_of_file(const char *filename) throw(error_obj)
{
	error_obj error;
	std::string trans;
	const std::string *tmp;
	const char err[] = "Old version of file";

	error.errorcode = 49;

	trans = err;
	if( (tmp = translation.get_key("ERROR_OLD_VERSION_OF_FILE")) )
		trans = *tmp;
	snprintf(error.msg,ERROR_OBJ_MSG_LEN,"%s (%s)",trans.c_str(),filename);
	throw error;
}

vartypes_check::vartypes_check()
{
	if(sizeof(int8) != 1)
	{
		printf("The int8 isn't 8 bits as it should, please correct it in file vartypes.h\n");
		exit(-1);
	}

	if(sizeof(int16) != 2)
	{
		printf("The int16 isn't 16 bits as it should, please correct it in file vartypes.h\n");
		exit(-1);
	}

	if(sizeof(int32) != 4)
	{
		printf("The int32 isn't 32 bits as it should, please correct it in file vartypes.h\n");
		exit(-1);
	}

	if(sizeof(int64) != 8)
	{
		printf("The int64 isn't 64 bits as it should, please correct it in file vartypes.h\n");
		exit(-1);
	}
}
