
/*
 *  variable typedef's and error messages
 *  keyfile_io - an easy way to read and write keyfiles
 *  Copyright (c) 2002-2005 by Mattias Hultgren <mattias_hultgren@tele2.se>
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

/*
News
----

v7  2006-02-25
--

	keyfile_io now supports lines with upto 5000 bytes (up from 500)

v6  2005-09-25 - 2005-11-06
--

	Added support for the escape sequence \"
	Moved ump_float from math2.h into this file
	Renamed error_lang -> translation
	Renamed ump_float -> floatx (x symbolice that the precision is choosed at compile time)

v1.1.4  2005-07-30
------

	Added an implementation of trunc using floor optional by -DDONT_HAVE_TRUNC

v1.1.3  2005-01-10
------

	added more standard functions for translatable error message
	added THROW_MACRO_FIX

v1.1.2  2004-10-17 - 2004-12-23
------

	added int64_max, int64_min, uin64_max, uint64_min
	small fixes
	added more standard functions for translatable error message
	fixed some compiler warnings

v1.1.1  2004-07-14 - 2004-09-01
------

	added more standard functions for translatable error message
	removed the Thrower field in error_obj
	added support to replace escape sequences in keyfile_io
	  known escape sequences are \\ \n \r \t
	added keyfile_io::write

v1.1.0  2004-06-30 - 2004-07-03
------

	merged keyfile_io.h and vartypes.h
	added some standard functions to ease up the work with translated error message

*/

#ifndef VARTYPES_H_
#define VARTYPES_H_

#define VARTYPES_H_VERSION "v7"
#define VARTYPES_H_DATE    "2002-12 - 2006-02"

#include <string>
#include <math.h>

class keyfile_io;

// load your translation into this object
extern keyfile_io translation;

// a gettext look-a-like macro
#define _(String) translation.get_text(String)


// this macro fixes so that code like throw (error_obj) { 3, "error" };  which causes ICE in gcc 3.4.x
// can be ported to THROW_MACRO_FIX( 3, "error" )  which works in gcc 3.4.x also
// (gcc 3.4.4 has fixed this bug...)
#define THROW_MACRO_FIX(nr,string) { \
                                     error_obj error; \
                                     error.errorcode = nr; \
                                     snprintf(error.msg,ERROR_OBJ_MSG_LEN,string); \
                                     throw error; \
                                   }




typedef unsigned char uint8;
typedef   signed char  int8;
const uint8 uint8_max = 0xff;
const uint8 uint8_min = 0x00;
const  int8  int8_max = 0x7f;
const  int8  int8_min = 0x80;

typedef unsigned short uint16;
typedef   signed short  int16;
const uint16 uint16_max = 0xffff;
const uint16 uint16_min = 0x0000;
const  int16  int16_max = 0x7fff;
const  int16  int16_min = 0x8000;

typedef unsigned int uint32;
typedef   signed int  int32;
const uint32 uint32_max = 0xffffffff;
const uint32 uint32_min = 0x00000000;
const  int32  int32_max = 0x7fffffff;
const  int32  int32_min = 0x80000000;

typedef unsigned long long int uint64;
typedef   signed long long int  int64;
const uint64 uint64_max = uint64(0) - 1;
const uint64 uint64_min = 0;
const  int64  int64_max = int64(uint64_max >> 1);
const  int64  int64_min = int64_max + 1;

#ifdef DONT_HAVE_TRUNC
double trunc(double value);
#endif
#ifdef DONT_HAVE_LOG2
double log2(double value);
#endif


#ifdef USE_FLOAT
	typedef float floatx;

	inline floatx sinx(floatx val) { return sinf(val); }
	inline floatx cosx(floatx val) { return cosf(val); }
	inline floatx tanx(floatx val) { return tanf(val); }
	inline floatx asinx(floatx val) { return asinf(val); }
	inline floatx acosx(floatx val) { return acosf(val); }
	inline floatx atanx(floatx val) { return atanf(val); }
	inline floatx sinhx(floatx val) { return sinhf(val); }
	inline floatx coshx(floatx val) { return coshf(val); }
	inline floatx tanhx(floatx val) { return tanhf(val); }
	inline floatx asinhx(floatx val) { return asinhf(val); }
	inline floatx acoshx(floatx val) { return acoshf(val); }
	inline floatx atanhx(floatx val) { return atanhf(val); }
	inline floatx atan2x(floatx v1,floatx v2) { return atan2f(v1,v2); }
	inline floatx hypotx(floatx v1,floatx v2) { return hypotf(v1,v2); }
	inline floatx truncx(floatx val) { return truncf(val); }
	inline floatx fabsx(floatx val) { return fabsf(val); }
	inline floatx logx(floatx val) { return logf(val); }
	inline floatx log2x(floatx val) { return log2f(val); }
	inline floatx log10x(floatx val) { return log10f(val); }
	inline floatx expx(floatx val) { return expf(val); }
	inline floatx sqrtx(floatx val) { return sqrtf(val); }
	inline floatx powx(floatx v1,floatx v2) { return powf(v1,v2); }
#endif
#ifdef USE_DOUBLE
	typedef double floatx;

	inline floatx sinx(floatx val) { return sin(val); }
	inline floatx cosx(floatx val) { return cos(val); }
	inline floatx tanx(floatx val) { return tan(val); }
	inline floatx asinx(floatx val) { return asin(val); }
	inline floatx acosx(floatx val) { return acos(val); }
	inline floatx atanx(floatx val) { return atan(val); }
	inline floatx sinhx(floatx val) { return sinh(val); }
	inline floatx coshx(floatx val) { return cosh(val); }
	inline floatx tanhx(floatx val) { return tanh(val); }
	inline floatx asinhx(floatx val) { return asinh(val); }
	inline floatx acoshx(floatx val) { return acosh(val); }
	inline floatx atanhx(floatx val) { return atanh(val); }
	inline floatx atan2x(floatx v1,floatx v2) { return atan2(v1,v2); }
	inline floatx hypotx(floatx v1,floatx v2) { return hypot(v1,v2); }
	inline floatx truncx(floatx val) { return trunc(val); }
	inline floatx fabsx(floatx val) { return fabs(val); }
	inline floatx logx(floatx val) { return log(val); }
	inline floatx log2x(floatx val) { return log2(val); }
	inline floatx log10x(floatx val) { return log10(val); }
	inline floatx expx(floatx val) { return exp(val); }
	inline floatx sqrtx(floatx val) { return sqrt(val); }
	inline floatx powx(floatx v1,floatx v2) { return pow(v1,v2); }
#endif
#ifdef USE_LONG_DOUBLE
	typedef long double floatx;

	inline floatx sinx(floatx val) { return sinl(val); }
	inline floatx cosx(floatx val) { return cosl(val); }
	inline floatx tanx(floatx val) { return tanl(val); }
	inline floatx asinx(floatx val) { return asinl(val); }
	inline floatx acosx(floatx val) { return acosl(val); }
	inline floatx atanx(floatx val) { return atanl(val); }
	inline floatx sinhx(floatx val) { return sinhl(val); }
	inline floatx coshx(floatx val) { return coshl(val); }
	inline floatx tanhx(floatx val) { return tanhl(val); }
	inline floatx asinhx(floatx val) { return asinhl(val); }
	inline floatx acoshx(floatx val) { return acoshl(val); }
	inline floatx atanhx(floatx val) { return atanhl(val); }
	inline floatx atan2x(floatx v1,floatx v2) { return atan2l(v1,v2); }
	inline floatx hypotx(floatx v1,floatx v2) { return hypotl(v1,v2); }
	inline floatx truncx(floatx val) { return truncl(val); }
	inline floatx fabsx(floatx val) { return fabsl(val); }
	inline floatx logx(floatx val) { return logl(val); }
	inline floatx log2x(floatx val) { return log2l(val); }
	inline floatx log10x(floatx val) { return log10l(val); }
	inline floatx expx(floatx val) { return expl(val); }
	inline floatx sqrtx(floatx val) { return sqrtl(val); }
	inline floatx powx(floatx v1,floatx v2) { return powl(v1,v2); }
#endif

extern const floatx PI, E;

const uint32 ERROR_OBJ_MSG_LEN = 1000;
struct error_obj
{
	uint32 errorcode;
	char msg[ERROR_OBJ_MSG_LEN];
};

// errorcode     this functions throws a translated error
/*     1     */ void Throw_Invalid_width_and_or_height  (void) throw(error_obj);
/*     1     */ void Throw_Invalid_matrix_width         (void) throw(error_obj);
/*     2     */ void Throw_Couldnt_get_memory           (void) throw(error_obj);
/*     4     */ void Throw_Couldnt_open_read_from_file  (const char *filename) throw(error_obj);
/*     6     */ void Throw_File_not_a_valid             (const char *type, const char *filename) throw(error_obj);
/*     7     */ void Throw_Couldnt_open_write_to_file   (const char *filename) throw(error_obj);
/*    16     */ void Throw_Unexpected                   (const char *unexpected) throw(error_obj);
/*    17     */ void Throw_Expected                     (const char *expected) throw(error_obj);
/*    18     */ void Throw_Divide_by_zero               (void) throw(error_obj);
/*    19     */ void Throw_Domain_error                 (const char *domain_error) throw(error_obj);
/*    20     */ void Throw_Illegal_use_of_reserved_key_word (const char *word) throw(error_obj);
/*    23     */ void Throw_Syntax_error                 (const char *syntax_error = 0) throw(error_obj);
/*    25     */ void Throw_Tried_to_change_a_constant   (void) throw(error_obj);
/*    26     */ void Throw_Couldnt_close_file           (const char *filename) throw(error_obj);
/*    29     */ void Throw_Data_type_is_unsupported_by  (const char *function) throw(error_obj);
/*    31     */ void Throw_Circle_reference_of_variable (const char *varname) throw(error_obj);
/*    32     */ void Throw_List_dont_match_for          (const char *function) throw(error_obj);
/*    32     */ void Throw_Matrix_size_dont_match       (void) throw(error_obj);
/*    33     */ void Throw_Overflow                     (void) throw(error_obj);
/*    41     */ void Throw_Name_is_not_unique           (const char *name) throw(error_obj);
/*    43     */ void Throw_No_solution_found            (void) throw(error_obj);
/*    48     */ void Throw_Invalid_format_in_file       (const char *filename) throw(error_obj);
/*    49     */ void Throw_Old_version_of_file          (const char *filename) throw(error_obj);




//   code            meaning
//    1              'Invalid layer#' or 'Invalid windowsize'
//    3              'Failed to init keyboard to raw-mode'
//    5  removed
//    8              'Update failure due to not in a graphicmode'
//    9              'Update failure due to screenbuffer-size'
//   10              'Couldn't get memory for the virtualscreen'
//   11              'Mode not supported by hardware'
//   12              'Mode not supported in linearmode by hardware'
//   13              'Mode not supported by vgapicture'
//   14              'Already using a graphicmode'
//   15              'Function-string empty'
//   20              'Illegal symbol/character' or 'Unknown symbol'
//   21              'Too many variables/constants' or 'Too many functions' or 'Too many variables/functions'
//   22              'Unknown function' or 'Unknown mark' or 'Unknown escape sequence' or 'Unknown symbol 'sym''
//   23              'Variable not found' or 'Variable/function not found' or 'Function not found'
//   27              'Variable isn't a value'
//   28              'Tried to change type of a type-locked variable'
//   30              'Empty bracket'
//   34              'Illegal type in matrix'
//   35              'Can't calculate invers-matrix (det equals to 0)'
//   36              'Illegal parameter values'
//   37              'No network found'
//   38              'Invalid picture-type'
//   39              'Unsuported compression method'
//   40              'Incomplete file (filename)'
//   42              'Variable is undeletable' or 'Can't delete variables in use'
//   44              'Variable isn't a 'Complex'-type' ...
//   45              'Variable has no name'
//   46  removed
//   47              'Unknown action code' or 'Unknown code' or 'Unknown format'
//   48              'break/continue must be used inside a while-statement'


// Note that the maximum fileline-width is 5000 bytes.

// comment lines must start with a '#', no blank steps before are allowed

// signs allowed in keyname's are a-z A-Z 0-9 _

// eg.  KEYNAME = "key value inside"
class keyfile_io
{
private:
	class keysDB
	{
	public:
		std::string name, value;

		keysDB() {}
		keysDB(keysDB &src);

		void operator=(keysDB &src);
	}*keys;
	uint32 nrofkeys;

	void operator=(const keyfile_io &src);

	void read(FILE *open_file, const char *filename) throw(error_obj);

public:
	// false is default
	bool replace_escape_sequences; // known escape sequences are  \\ \n \t \r \"

	keyfile_io();
	keyfile_io(const char *filename) throw(error_obj);
	keyfile_io(const keyfile_io &src) throw(error_obj);
	~keyfile_io();

	// empties the object
	void reset(void);

	// sets a key to a special value, if the key is found it will be replaced
	void set_key(const std::string &keyname,const std::string &value) throw(error_obj);

	// if keyname wasn't found keyname is returned
	const char* get_text( const char *keyname );

	// if keyname wasn't found the fail_pointer is returned
	const std::string* get_key(const std::string &keyname, const std::string *fail_pointer=0);
	const char* get_key_c_str(const std::string &keyname, const char *fail_pointer=0);

	// this will remove the keyname, if it's not found nothing happens
	void remove_key(const std::string &keyname) throw(error_obj);

	// this doesn't call reset(), the keys in the file will be added to the ones already present
	void read(const char *filename) throw(error_obj);

	// note that intern_file isn't a filename but a file contant (NULL-terminated)
	void read_intern(const char *intern_file) throw(error_obj);

	void write(const char *filename) throw(error_obj);
};



#endif // VARTYPES_H_
